## This file generated by InlineX::C2XS (version 0.25) using Inline::C (version 0.76)
package Math::GMPz::V;
use strict;
use warnings;

require Exporter;
*import = \&Exporter::import;
require DynaLoader;

our $VERSION = '0.53';
#$VERSION = eval $VERSION;
Math::GMPz::V->DynaLoader::bootstrap($VERSION);

@Math::GMPz::V::EXPORT = ();
@Math::GMPz::V::EXPORT_OK = ();

sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking

1;
